import {
  EntityHealthComponent,
  system,
  world,
  EquipmentSlot,
  EntityInventoryComponent,
  ItemStack
} from "@minecraft/server";
import functions from "./block/functions";
import ItemDurability from "./block/durability";

const protectedPlayers = new Map();

system.runInterval(async () => {
  for (const player of world.getPlayers()) {
    functions.getItemsWithTag(player, "minecraft:can_block").forEach((item) => {
      const playerInventoryComponentContainer = player.getComponent(
        EntityInventoryComponent.componentId
      ).container;
      item.item.setLore(["§9Able to Block"]);
      playerInventoryComponentContainer.setItem(item.slot, item.item);
    });
    const weapon = functions.findWeapon(player);

    if (functions.getWeapon(player, true)?.typeId == "minecraft:shield")
      continue;
    if (!weapon || !player.isSneaking) {
      protectedPlayers.delete(player.id);
      continue;
    } else if (!protectedPlayers.get(player.id))
      protectedPlayers.set(player.id, weapon);
    player.addEffect("weakness", 20, { amplifier: 2, showParticles: false });
    player.playAnimation("animation.player.weapon_blocking");
  }
});

world.afterEvents.entityHurt.subscribe(
  (event) => {
    const { hurtEntity: player, damage, damageSource: source } = event;
    const { damagingEntity: entity, damagingProjectile: projectile } = source;
    const protectedPlayer = protectedPlayers.get(player.id);

    if (!protectedPlayer) return;
    if (isInBack(player.getRotation().y, entity.getRotation().y, 50)) return;
    const itemStack = functions
      .getEquipmentComponent(player)
      ?.getEquipment(EquipmentSlot.Mainhand);
    const damagedItem = ItemDurability.damageItem(itemStack, 1);

    if (!damagedItem) {
      functions
        .getEquipmentComponent(player)
        .setEquipment(EquipmentSlot.Mainhand, new ItemStack("air"));
      player.playSound("random.break");
      return;
    }
    functions
        .getEquipmentComponent(player)
        .setEquipment(EquipmentSlot.Mainhand, itemStack);
    const healthComponent = player.getComponent(
      EntityHealthComponent.componentId
    );
    const reduction = functions.calculateReduction(
      damage,
      entity || projectile,
      !!projectile
    );

    player.playSound("item.weapon.block");
    player.teleport(player.location);
    system.run(() => {
      healthComponent.setCurrentValue(healthComponent.currentValue + reduction);
    });
  },
  { entityTypes: ["minecraft:player"] }
);

function isInBack(rot1, rot2, threshold) {
  const adjustedYourRotation = ((rot2 + 180) % 360) - 180;
  const adjustedEntityRotation = ((rot1 + 180) % 360) - 180;
  const rotationDifference = Math.abs(
    adjustedYourRotation - adjustedEntityRotation
  );

  return rotationDifference <= threshold;
}

function splitter(item) {
    let splitted = item.split("_")
    if(splitted.includes("lucernehammer")) {
        return("lucernehammer")
    } else if(splitted.includes("heavywarhammer")) {
        return("heavywarhammer")
    } else if(splitted.includes("bo")) {
        return("bo")
    }else {
        return "blud"
    }
}

world.afterEvents.entityHitEntity.subscribe((eyy) => {
    let damager = eyy.damagingEntity
    let damaged = eyy.hitEntity
    
    if (damager.typeId === "minecraft:player") {;
        let equipment = damager.getComponent("minecraft:equippable");
        let mainhand = equipment?.getEquipmentSlot(`Mainhand`)?.getItem()
        if(mainhand == undefined) return;
        let check = splitter(mainhand.typeId)
        if(check != "blud" && check != "bo") {
            let equipmentOther = damaged.getComponent("minecraft:equippable");
            let offhand = equipmentOther?.getEquipmentSlot(`Offhand`)?.getItem()
            if(offhand == undefined) return;
            var NewItem = offhand
            if(NewItem.getComponent("durability") != undefined) {
                damaged.getComponent('equippable')?.setEquipment(EquipmentSlot.Offhand, undefined)
            }
        } else if(check == "bo") {
            let chance = Math.floor(Math.random()*10)
            if(chance == 5) {
                damaged.addEffect('slowness', 40, {amplifier: 2,})
            } else if(chance == 5) {
                damaged.addEffect('nausea', 40, {amplifier: 2,})
            } else if(chance == 5) {
                damaged.addEffect('weakness', 40, {amplifier: 2,})
            }
        }
    }
});

system.runInterval(() => {
    for(const player of world.getPlayers()) {
        let equipment = player.getComponent("minecraft:equippable");
        let mainhand = equipment?.getEquipmentSlot(EquipmentSlot.Mainhand)?.getItem()
        let offhand = equipment?.getEquipmentSlot(EquipmentSlot.Offhand)?.getItem()
        let tags = mainhand.getTags()
        if(tags.includes('minecraft:is_twohanded') && offhand != undefined) {
            player.addEffect("weakness", 100, { amplifier: 1, showParticles: false })
        } else if(tags.includes('minecraft:is_twohanded_two') && offhand != undefined) {
            player.addEffect("slowness", 100, { amplifier: 0, showParticles: false })
            player.addEffect("weakness", 100, { amplifier: 1, showParticles: false })
        }
    }
},20)
